.\" This used to cause a hang due to a bug in register expansion.
.TL
Pascal-F Verifier Internal Design Document

``Jcode'' Interface Control Document
.AF "FACC / Palo Alto"
.AU "Scott Johnson" JBN "" "Software Technology"
.PH "''Pascal-F Verifier Interface Control Document'Page \\\\nP'"
.PF "'Draft 1.23'"Jcode" Interface Control Document'12/13/82'"
.MT 4
.SA 1
.de ES
.nf
.in +\n(Sin
..
.de EE
.SP 1
..
.de (d
.nf
..
.de )d
.fi
.nr ss \\n(Si    \" Save the standard indent
.nr Si \\n(.l-\\n(dl/2/1m  \" Change standard indent to center diversion
.DS 1 0          \" begin indented display
.DE              \" end display
.nr Si \\n(ss    \" restore the standard indent
..
.PH "'J-code Documentation   Version 1.22''Page \\\\nP'"
.PF "''Draft of 12/13/82''"
.H 1 "Introduction"
.(d
<j-code> ::= <j-unit>*
.)d
.ES
<j-statement> ::= <simple statement> | <throw statement>
                | <catch statement> | <rein statement>
                | <renew statement> | <reout statement>
.EE
